################################################################################
# This script is an license and program sector for ArcGIS Desktop
# 
#  Script name: ArcGIS_Desktop_Selector_en.pyw
#  Release 2.4
#  Written by unknown 12th January 2018
#  Modified by Jeremy Saunders (jeremy@jhouseconsulting.com) 24th February 2024
#
#  Enhanced to run with Python 2.7 and above and 3.6 and above.
#
#  Note that to run ArcGIS Administrator users must have either local Administrator permissions,
#  or full control of the following registry key structures:
#  - HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\ESRI
#  - HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Classes\CLSID\{E6BDAA76-4D35-11D0-98BE-00805F7CED21}
#
# Note that on a 64-bit machine there are two Python environments installed with ArcGIS Desktop.
# For example, for ArcGIS Desktop 10.5 the following two Python environments exist.
# - C:\Python27\ArcGIS10.5
# - C:\Python27\ArcGISx6410.5
# Interestingly the Windows Shell/Context Menu integration uses C:\Python27\ArcGISx6410.5.
# However, as ArcGIS Desktop programs are 32-bit, they use C:\Python27\ArcGIS10.5. It can
# be confusing. So if you are going to use this menu selector to install Python modules,
# it must be run with the "C:\Python27\ArcGISx6410.5\pythonw.exe" interpreter to ensure
# that the modules are correctly installed for use with the ArcGIS Desktop programs.
#
################################################################################

# Set these variables

# Pass the Version as a argument. If you set this to False, you must set a version number
# below
VersionAsArgument = True

# Set the Version of ArcGIS Desktop in the format of major.minor if not passing it as an
# argument
version = "10.8"

# Set the minutes that this window will stay open for before closing
# Set to 0 to disable the timer
MinutesBeforeClosing = 720

# Set to True to give the users the ability to install extra Python Modules
InstallPythonModules = True

# This variable contains a list of extra Python modules to install
PythonModulesToInstall = ['pyodbc','PyYAML','Pillow']

# Set to True to give the users the ability to launch File Explorer
AddFileExplorer = True

# Set to True to give the users the ability to delete the Normal.mxt file
DeleteNormalmxt = True

# Set to True to enable the ArcMap startup log. This will write an ArcMap.log file to the
# current directory
EnableArcMapStartupLog = False

# Set to True to give the users the ability to copy existing Geodatabase Connections
CopyGeodatabaseConnections = False
# C:\Users\<username>\AppData\Roaming\ESRI\Desktop<release#>\ArcCatalog
# os.path.join(os.getenv("APPDATA"), r"ESRI\Desktop"+version+"\ArcCatalog")

# Set to True to give the users the ability to copy existing Coordinate Systems and
# projection files
CopyCoordinateSystems = False
# C:\Users\<username>\AppData\Roaming\ESRI\Desktop<release#>\ArcMap\Coordinate Systems
# os.path.join(os.getenv("APPDATA"), r"ESRI\Desktop"+version+"\ArcMap\Coordinate Systems")
# Note that this includes custom projection files (.prj)

# Set to True to enable screen resolution (width and height) output in pixels to
# assist with the geometry of the main window.
EnableScreenResolutionDebugging = False

################################################################################

import os, sys, subprocess

if VersionAsArgument:
    if (len(sys.argv) == 2):
        version = sys.argv[1]
    else:
        print("Version argument missing")
        sys.exit(0)

if sys.version_info[0] == 3:
    import winreg
    from tkinter import *
    from tkinter import messagebox
else:
    import _winreg as winreg
    from Tkinter import *
    import tkMessageBox

# Derive the ArcGIS Installation directory from the registry
try:
    rk = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,"SOFTWARE\\Wow6432Node\\ESRI\\Desktop" + version)
    tuple = winreg.QueryValueEx(rk,"InstallDir")
    Path = tuple[0]                     
    winreg.CloseKey(rk)
except:
    print("Registry key not accessible!")

ArcGISDesktopPath = Path + "bin\\"
exists = os.path.isdir(ArcGISDesktopPath)
if not exists:
    print("ArcGIS Desktop is not installed")
    sys.exit(0)

try:
    rk = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,"SOFTWARE\\Wow6432Node\\ESRI\\ArcGIS")
    tuple = winreg.QueryValueEx(rk,"InstallDir")
    Path = tuple[0]                     
    winreg.CloseKey(rk)
except:
    print("Registry key not accessible!")

ArcGISDesktopCommonPath = Path + "bin\\"
exists = os.path.isdir(ArcGISDesktopCommonPath)
if not exists:
    print("ArcGIS Desktop is not installed")
    sys.exit(0)

# Set the name of the icon file to use. If not found it will use the default Tkinter icon.
current_directory = os.path.dirname(os.path.abspath(__file__))
iconfile = current_directory  + r"\ArcGIS_Desktop_Product_Icon.ico"
useicon = False
if os.path.exists(iconfile):
    useicon = True

# Derive the path to the Normal.mxt file
normalmxtfile = os.getenv('APPDATA') + r"\ESRI\Desktop" + version + r"\ArcMap\Templates\Normal.mxt"

# Use the %USERPROFILE% as the working folder
workingfolder = os.getenv('USERPROFILE')

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# To avoid the GUI from becoming unresponsive due to a "thread lock" issue, we can utilize the after()
# method of the tkinter window to schedule the window to close after a specified duration without
# blocking the main event loop. We define the close_after() function, which uses the after() method to
# schedule the close_window() function to be called after the specified delay. This approach allows the
# main event loop to continue running and keeps the GUI responsive. It is good practice to define these
# outside of the mainloop to ensure it's only called once and not with each iteration of the loop.

def close_window():
    root.destroy()

def close_after(delay):
    root.after(delay, close_window)

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

root = Tk()
# Preventing the root window to change size
root.resizable(0,0)
# Allowing the root window to change size
#root.resizable(True, True)
root.title("ArcGIS Desktop " + version + " Selector")
if useicon:
    root.iconbitmap(iconfile)

computername = os.environ['COMPUTERNAME']
l1 = Label(root, text='You are running this on ' + computername ,relief = FLAT, fg = 'gray40' , anchor = "w", font = ('Arial', 8, 'bold'))
l1.grid()

l2 = Label(root, text='Choose the type of license:' ,relief = FLAT, fg = 'gray20' , anchor = "w", font = ('Arial', 10, 'bold'), width = 25)
l2.grid()
choices = ["Basic", "Standard", "Advanced"]
list = Listbox(root, height=3, width=20, selectmode = SINGLE)
list.grid()
for item in choices:
    list.insert(END, item)

foo = IntVar()

l3 = Label(root, text='Choose the application:' ,relief = FLAT, fg = 'gray20' , anchor = "w", font = ('Arial', 10, 'bold'), width = 25)
l3.grid()

rb1 = Radiobutton(root, text = "ArcMap", variable=foo, value=0, anchor = "w", width = 25, padx=25)
rb1.grid()

rb2 = Radiobutton(root, text = "ArcCatalog", variable=foo, value=1, anchor = "w", width = 25, padx=25)
rb2.grid()

rb3 = Radiobutton(root, text = "ArcGlobe", variable=foo, value=2, anchor = "w", width = 25, padx=25)
rb3.grid()

rb4 = Radiobutton(root, text = "ArcScene", variable=foo, value=3, anchor = "w", width = 25, padx=25)
rb4.grid()

rb5 = Radiobutton(root, text = "ArcGIS Administrator", variable=foo, value=4, anchor = "w", width = 25, padx=25)
rb5.grid()

if DeleteNormalmxt:
    rb6 = Radiobutton(root, text = "Delete the Normal.mxt file", variable=foo, value=5, anchor = "w", width = 25, padx=25)
    rb6.grid()
    if not os.path.exists(normalmxtfile):
        rb6.configure(state = DISABLED)

if (InstallPythonModules and PythonModulesToInstall):
    rb7 = Radiobutton(root, text = "Install the following modules:", variable=foo, value=6, anchor = "w", width = 25, padx=25)
    rb7.grid()
    for item in PythonModulesToInstall:
        l4 = Label(root, text="- " + item , justify=LEFT, anchor="w", padx=45)
        l4.grid(sticky=W)

if AddFileExplorer:
    rb8 = Radiobutton(root, text = "File Explorer", variable=foo, value=7, anchor = "w", width = 25, padx=25)
    rb8.grid()

def get_item_selected(slecteditem):
    if (slecteditem == 0):   
        os.environ['ESRI_SOFTWARE_CLASS']='Viewer'
    if (slecteditem == 1):        
        os.environ['ESRI_SOFTWARE_CLASS']='Editor'
    if (slecteditem == 2):        
        os.environ['ESRI_SOFTWARE_CLASS']='Professional'

def view_selected():
    flavour = str("")
    item = int(list.curselection()[0])
    number = foo.get()
    if (number == 0):
        get_item_selected(item)
        if not EnableArcMapStartupLog:
            flavour = str(ArcGISDesktopPath + "\ArcMap.exe")
        else:
            flavour = str(ArcGISDesktopPath + "\ArcMap.exe" + " /log")
    if (number == 1):
        get_item_selected(item)
        flavour = str(ArcGISDesktopPath + "\ArcCatalog.exe")
    if (number == 2):
        get_item_selected(item)
        flavour = str(ArcGISDesktopPath + "\ArcGlobe.exe")
    if (number == 3):
        get_item_selected(item)
        flavour = str(ArcGISDesktopPath + "\ArcScene.exe")
    if (number == 4):
        get_item_selected(item)
        flavour = str(ArcGISDesktopCommonPath + "\ArcGISAdmin.exe")
    if DeleteNormalmxt:
        if (number == 5):
            if os.path.exists(normalmxtfile):
                os.remove(normalmxtfile)
            if not os.path.exists(normalmxtfile):
                rb6.configure(state = DISABLED)
    if (InstallPythonModules and PythonModulesToInstall):
        if (number == 6):
            for item in PythonModulesToInstall:
                output = ""
                command = [sys.executable, '-m', 'pip', 'install', item]
                if (sys.version_info[0] == 2 or (sys.version_info[0] == 3 and sys.version_info[1] < 5)):
                    try:
                        output = subprocess.check_output(command,universal_newlines=True)
                    except subprocess.CalledProcessError as e:
                        output = e.output
                    except:
                        output = "Unexpected error"
                    #print(output)
                if ((sys.version_info[0] == 3 and sys.version_info[1] >= 5) or sys.version_info[0] > 3):
                    process = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
                    if(len(process.stdout.strip()) > 0):
                        output = process.stdout
                    if(len(process.stderr.strip()) > 0):
                        output = process.stderr
                    if(len(process.stdout.strip()) == 0 and len(process.stderr.strip()) == 0):
                        output = "Unexpected error"
                    #print(output)
    if AddFileExplorer:
        if (number == 7):        
            flavour = str("C:\Windows\explorer.exe /n ,::{20D04FE0-3AEA-1069-A2D8-08002B30309D}")

    if flavour.strip():
        if os.path.exists(workingfolder):
            # Change the current working Directory to a user specific folder before launching
            os.chdir(workingfolder)
            #if sys.version_info[0] == 3:
                # messagebox.showinfo("Working Folder", workingfolder)
            #else:
                # tkMessageBox.showinfo("Working Folder", workingfolder)
        subprocess.Popen(flavour)

btn = Button(root, text = "Launch Selection", command = view_selected, width = 20, font = ('Arial', 10, 'bold') )
btn.grid()

list.selection_set(0,)

l5 = Label(root, text='Basic ArcGIS Desktop formerly ArcView' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
l5.grid()
l6 = Label(root, text='Standard ArcGIS Desktop formerly ArcEditor' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
l6.grid()
l7 = Label(root, text='Advanced ArcGIS Desktop formerly ArcInfo' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
l7.grid()

if (MinutesBeforeClosing != 0):
    MinutesBeforeClosingText = StringVar()
    MinutesBeforeClosingText.set('after ' + str(MinutesBeforeClosing) + ' minutes.')
    l8 = Label(root, text= 'This window will automatically close' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l8.grid()
    l9 = Label(root, textvariable = MinutesBeforeClosingText ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l9.grid()

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# According to your system settings such as fonts and screen resolution, 
# you might need to adjust the root.geometry(width,height). It's typically
# the height that becomes a problem with higher resolution (4K) screens.
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# Getting screen's height in pixels
height = root.winfo_screenheight()
# Getting screen's width in pixels
width = root.winfo_screenwidth()
if EnableScreenResolutionDebugging:
    ScreenResolution = StringVar()
    ScreenResolution.set('width x height = ' + str(width) + ' x ' + str(height) + ' (in pixels)')
    l10 = Label(root, text='Screen Resolution:' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l10.grid()
    l11 = Label(root, textvariable = ScreenResolution ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l11.grid()

height = str("375")
if AddFileExplorer:
    height = str(int(height) + 25)
if DeleteNormalmxt:
    height = str(int(height) + 25)
if (InstallPythonModules and PythonModulesToInstall):
    height = str(int(height) + ((len(PythonModulesToInstall) + 1) * 25))
if (MinutesBeforeClosing != 0):
    height = str(int(height) + 50)
if EnableScreenResolutionDebugging:
    height = str(int(height) + 50)
root.geometry("320x" + height)

# Automatically close the window after a specified time
mytimer = int(MinutesBeforeClosing * 60 * 1000)
if (MinutesBeforeClosing != 0):
    close_after(mytimer)

root.mainloop()
